"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var rxjs_1 = require('rxjs');
var ReactiveEvent_1 = require('./ReactiveEvent');
var ReactiveState_1 = require('./ReactiveState');
var SubjectScheduler_1 = require('./SubjectScheduler');
// NOTE: we don't type the parameter because it would mean that you would have
//       to always provide that type information when declaring a command, which
//       is undesirable.
var ReactiveCommand = (function (_super) {
    __extends(ReactiveCommand, _super);
    function ReactiveCommand(owner, executeAction, canExecute, scheduler, errorScheduler) {
        var _this = this;
        _super.call(this, errorScheduler);
        this.owner = owner;
        this.executeAction = executeAction;
        this.isExecutingSubject = new rxjs_1.BehaviorSubject(false);
        this.isExecutingScheduledSubject = new SubjectScheduler_1.SubjectScheduler(scheduler, undefined, this.isExecutingSubject);
        this.add(this.isExecutingScheduledSubject);
        if (canExecute == null) {
            canExecute = rxjs_1.Observable.of(true);
        }
        this.canExecuteObservable = rxjs_1.Observable
            .combineLatest(this.isExecuting, canExecute, function (ie, ce) { return ie === false && ce; });
        this.add(this.changing
            .subscribe(function (x) {
            _this.notifyPropertyChanged(function () { return new ReactiveEvent_1.ReactiveEvent(_this, {
                param: x.value.param,
                result: _this.executeAction.apply(_this, [x.value.param]),
            }); });
        }, this.thrownErrorsHandler.next));
        this.add(this.changed
            .subscribe(function (x) {
            _this.isExecutingScheduledSubject.next(false);
        }, this.thrownErrorsHandler.next));
    }
    Object.defineProperty(ReactiveCommand.prototype, "isExecuting", {
        get: function () {
            return this.isExecutingScheduledSubject
                .asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ReactiveCommand.prototype, "results", {
        get: function () {
            return this.changed
                .filter(function (x) { return x != null && x.value != null; })
                .map(function (x) { return x.value.result; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ReactiveCommand.prototype, "canExecute", {
        get: function () {
            return this.canExecuteObservable;
        },
        enumerable: true,
        configurable: true
    });
    ReactiveCommand.prototype.execute = function (param) {
        var _this = this;
        if (this.isExecutingSubject.value === true) {
            throw 'Command Execution is Already in Progress';
        }
        this.isExecutingScheduledSubject.next(true);
        this.notifyPropertyChanging(function () { return new ReactiveEvent_1.ReactiveEvent(_this, { param: param, result: null }); });
    };
    return ReactiveCommand;
}(ReactiveState_1.ReactiveState));
exports.ReactiveCommand = ReactiveCommand;

//# sourceMappingURL=ReactiveCommand.js.map
