"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var ReactiveEvent_1 = require('./ReactiveEvent');
var ReactiveState_1 = require('./ReactiveState');
function toReactiveList(owner, errorScheduler) {
    var thisArg = this;
    return new ReactiveList(owner, thisArg, errorScheduler);
}
Array.prototype.toReactiveList = toReactiveList;
(function (ReactiveListChangeAction) {
    ReactiveListChangeAction[ReactiveListChangeAction["Add"] = 0] = "Add";
    ReactiveListChangeAction[ReactiveListChangeAction["Remove"] = 1] = "Remove";
    ReactiveListChangeAction[ReactiveListChangeAction["Replace"] = 2] = "Replace";
    ReactiveListChangeAction[ReactiveListChangeAction["Move"] = 3] = "Move";
    ReactiveListChangeAction[ReactiveListChangeAction["Reset"] = 4] = "Reset";
})(exports.ReactiveListChangeAction || (exports.ReactiveListChangeAction = {}));
var ReactiveListChangeAction = exports.ReactiveListChangeAction;
var ReactiveList = (function (_super) {
    __extends(ReactiveList, _super);
    function ReactiveList(owner, items, errorScheduler) {
        if (items === void 0) { items = []; }
        _super.call(this, errorScheduler);
        this.owner = owner;
        this.items = items;
    }
    ReactiveList.prototype.wrapArrayFunction = function (func, args, eventArgs) {
        var _this = this;
        var sub = this.changing.subscribe(function (x) {
            if (x.value === eventArgs) {
                func.apply(x.source.items, args);
                x.source.notifyPropertyChanged(function () { return new ReactiveEvent_1.ReactiveEvent(x.source, x.value); });
                sub.unsubscribe();
            }
        });
        this.notifyPropertyChanging(function () { return new ReactiveEvent_1.ReactiveEvent(_this, eventArgs); });
    };
    ReactiveList.prototype.get = function (index) {
        return this.items[index];
    };
    ReactiveList.prototype.set = function (index, value) {
        var _this = this;
        var eventArgs = {
            action: ReactiveListChangeAction.Replace,
            newItems: [value],
            newStartingIndex: index,
            oldItems: this.items.slice(index, index + 1),
            oldStartingIndex: index,
        };
        this.wrapArrayFunction(function () { _this.items[index] = value; }, null, eventArgs);
    };
    ReactiveList.prototype.asArray = function () {
        return this.items;
    };
    ReactiveList.prototype.toArray = function () {
        return this.items.slice();
    };
    ReactiveList.prototype.clear = function () {
        this.reset();
    };
    ReactiveList.prototype.reset = function () {
        var items = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            items[_i - 0] = arguments[_i];
        }
        this.splice.apply(this, [0, this.items.length].concat(items));
    };
    Object.defineProperty(ReactiveList.prototype, "length", {
        // Array<TValue> Implementation
        get: function () {
            return this.items.length;
        },
        enumerable: true,
        configurable: true
    });
    ReactiveList.prototype.push = function () {
        var items = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            items[_i - 0] = arguments[_i];
        }
        this.wrapArrayFunction(this.items.push, arguments, {
            action: ReactiveListChangeAction.Add,
            newItems: items,
            newStartingIndex: this.items.length,
        });
    };
    ReactiveList.prototype.pop = function () {
        this.wrapArrayFunction(this.items.pop, arguments, {
            action: ReactiveListChangeAction.Remove,
            oldItems: this.items.slice(-1),
            oldStartingIndex: this.items.length - 1,
        });
    };
    ReactiveList.prototype.concat = function () {
        var items = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            items[_i - 0] = arguments[_i];
        }
        return this.items.concat.apply(this.items, arguments);
    };
    ReactiveList.prototype.join = function (separator) {
        return this.items.join.apply(this.items, arguments);
    };
    ReactiveList.prototype.reverse = function () {
        this.wrapArrayFunction(this.items.reverse, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    };
    ReactiveList.prototype.shift = function () {
        this.wrapArrayFunction(this.items.shift, arguments, {
            action: ReactiveListChangeAction.Remove,
            oldItems: this.items.slice(0, 1),
            oldStartingIndex: 0,
        });
    };
    ReactiveList.prototype.slice = function (start, end) {
        return this.items.slice.apply(this.items, arguments);
    };
    ReactiveList.prototype.sort = function (compareFn) {
        this.wrapArrayFunction(this.items.sort, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    };
    ReactiveList.prototype.splice = function () {
        // we have to reset here because this could be an add, a remove, a replace, or some combination
        this.wrapArrayFunction(this.items.splice, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    };
    ReactiveList.prototype.unshift = function () {
        var items = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            items[_i - 0] = arguments[_i];
        }
        this.wrapArrayFunction(this.items.unshift, arguments, {
            action: ReactiveListChangeAction.Add,
            newItems: items,
            newStartingIndex: 0,
        });
    };
    ReactiveList.prototype.indexOf = function (searchElement, fromIndex) {
        return this.items.indexOf.apply(this.items, arguments);
    };
    ReactiveList.prototype.lastIndexOf = function (searchElement, fromIndex) {
        return this.items.lastIndexOf.apply(this.items, arguments);
    };
    ReactiveList.prototype.every = function (callbackfn, thisArg) {
        return this.items.every.apply(this.items, arguments);
    };
    ReactiveList.prototype.some = function (callbackfn, thisArg) {
        return this.items.some.apply(this.items, arguments);
    };
    ReactiveList.prototype.forEach = function (callbackfn, thisArg) {
        return this.items.forEach.apply(this.items, arguments);
    };
    ReactiveList.prototype.map = function (callbackfn, thisArg) {
        return this.items.map.apply(this.items, arguments);
    };
    ReactiveList.prototype.filter = function (callbackfn, thisArg) {
        return this.items.filter.apply(this.items, arguments);
    };
    ReactiveList.prototype.reduce = function (callbackfn, initialValue) {
        return this.items.reduce.apply(this.items, arguments);
    };
    ReactiveList.prototype.reduceRight = function (callbackfn, initialValue) {
        return this.items.reduceRight.apply(this.items, arguments);
    };
    // Array<T> ES6 Shims
    ReactiveList.prototype.find = function (predicate, thisArg) {
        return this.items.find.apply(this.items, arguments);
    };
    ReactiveList.prototype.findIndex = function (predicate, thisArg) {
        return this.items.findIndex.apply(this.items, arguments);
    };
    ReactiveList.prototype.fill = function (value, start, end) {
        // we have to reset here because this could be an add, a replace, or both
        this.wrapArrayFunction(this.items.fill, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    };
    ReactiveList.prototype.copyWithin = function (target, start, end) {
        // we have to reset here because this could be an add, a replace, or both
        this.wrapArrayFunction(this.items.copyWithin, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    };
    ReactiveList.prototype.entries = function () {
        return this.items.entries.apply(this.items, arguments);
    };
    ReactiveList.prototype.keys = function () {
        return this.items.keys.apply(this.items, arguments);
    };
    ReactiveList.prototype.values = function () {
        return this.items.values.apply(this.items, arguments);
    };
    return ReactiveList;
}(ReactiveState_1.ReactiveState));
exports.ReactiveList = ReactiveList;

//# sourceMappingURL=ReactiveList.js.map
