"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var ReactiveState_1 = require('./ReactiveState');
var ReactiveEvent_1 = require('./ReactiveEvent');
var ReactiveProperty_1 = require('./ReactiveProperty');
var ReactiveCommand_1 = require('./ReactiveCommand');
var ReactiveList_1 = require('./ReactiveList');
var ReactiveObject = (function (_super) {
    __extends(ReactiveObject, _super);
    function ReactiveObject(errorScheduler) {
        _super.call(this, errorScheduler);
        this.members = [];
    }
    ReactiveObject.prototype.getMemberName = function (member) {
        if (member.name == null) {
            this.resolvePropertyNames();
        }
        return member.name;
    };
    ReactiveObject.prototype.resolvePropertyNames = function () {
        var _this = this;
        Object
            .getOwnPropertyNames(this)
            .map(function (name) { return ({ name: name, member: _this[name] }); })
            .filter(function (x) { return x.member != null && x.member.isReactive === true && x.member.name == null; })
            .forEach(function (x) {
            x.member.name = x.name;
        });
    };
    ReactiveObject.prototype.registerMember = function (member) {
        var _this = this;
        this.add(member.changing
            .subscribe(function (x) {
            _this.notifyPropertyChanging(function () { return new ReactiveEvent_1.ReactiveEvent(_this, {
                member: x.source,
                memberName: _this.getMemberName(x.source),
            }); });
        }, this.thrownErrorsHandler.next));
        this.add(member.changed
            .subscribe(function (x) {
            _this.notifyPropertyChanged(function () { return new ReactiveEvent_1.ReactiveEvent(_this, {
                member: x.source,
                memberName: _this.getMemberName(x.source),
            }); });
        }, this.thrownErrorsHandler.next));
        this.members.push(member);
    };
    ReactiveObject.prototype.propertyFrom = function (source, initialValue, errorScheduler) {
        var prop = new ReactiveProperty_1.ReactiveStreamProperty(source, this, initialValue, errorScheduler);
        this.registerMember(prop);
        return prop;
    };
    ReactiveObject.prototype.property = function (initialValue, scheduler, errorScheduler) {
        var prop = new ReactiveProperty_1.ReactiveValueProperty(this, initialValue, scheduler, errorScheduler);
        this.registerMember(prop);
        return prop;
    };
    ReactiveObject.prototype.command = function (executeAction, canExecute, errorScheduler) {
        var cmd = new ReactiveCommand_1.ReactiveCommand(this, executeAction, canExecute, errorScheduler);
        this.registerMember(cmd);
        return cmd;
    };
    ReactiveObject.prototype.list = function (items, errorScheduler) {
        var list = new ReactiveList_1.ReactiveList(this, items, errorScheduler);
        this.registerMember(list);
        return list;
    };
    return ReactiveObject;
}(ReactiveState_1.ReactiveState));
exports.ReactiveObject = ReactiveObject;

//# sourceMappingURL=ReactiveObject.js.map
