import { Observable, BehaviorSubject } from 'rxjs';
import { ReactiveEvent } from './ReactiveEvent';
import { ReactiveState } from './ReactiveState';
import { SubjectScheduler } from './SubjectScheduler';
// NOTE: we don't type the parameter because it would mean that you would have
//       to always provide that type information when declaring a command, which
//       is undesirable.
export class ReactiveCommand extends ReactiveState {
    constructor(owner, executeAction, canExecute, scheduler, errorScheduler) {
        super(errorScheduler);
        this.owner = owner;
        this.executeAction = executeAction;
        this.isExecutingSubject = new BehaviorSubject(false);
        this.isExecutingScheduledSubject = new SubjectScheduler(scheduler, undefined, this.isExecutingSubject);
        this.add(this.isExecutingScheduledSubject);
        if (canExecute == null) {
            canExecute = Observable.of(true);
        }
        this.canExecuteObservable = Observable
            .combineLatest(this.isExecuting, canExecute, (ie, ce) => ie === false && ce);
        this.add(this.changing
            .subscribe(x => {
            this.notifyPropertyChanged(() => new ReactiveEvent(this, {
                param: x.value.param,
                result: this.executeAction.apply(this, [x.value.param]),
            }));
        }, this.thrownErrorsHandler.next));
        this.add(this.changed
            .subscribe(x => {
            this.isExecutingScheduledSubject.next(false);
        }, this.thrownErrorsHandler.next));
    }
    get isExecuting() {
        return this.isExecutingScheduledSubject
            .asObservable();
    }
    get results() {
        return this.changed
            .filter(x => x != null && x.value != null)
            .map(x => x.value.result);
    }
    get canExecute() {
        return this.canExecuteObservable;
    }
    execute(param) {
        if (this.isExecutingSubject.value === true) {
            throw 'Command Execution is Already in Progress';
        }
        this.isExecutingScheduledSubject.next(true);
        this.notifyPropertyChanging(() => new ReactiveEvent(this, { param, result: null }));
    }
}

//# sourceMappingURL=ReactiveCommand.js.map
