import { ReactiveEvent } from './ReactiveEvent';
import { ReactiveState } from './ReactiveState';
function toReactiveList(owner, errorScheduler) {
    const thisArg = this;
    return new ReactiveList(owner, thisArg, errorScheduler);
}
Array.prototype.toReactiveList = toReactiveList;
export var ReactiveListChangeAction;
(function (ReactiveListChangeAction) {
    ReactiveListChangeAction[ReactiveListChangeAction["Add"] = 0] = "Add";
    ReactiveListChangeAction[ReactiveListChangeAction["Remove"] = 1] = "Remove";
    ReactiveListChangeAction[ReactiveListChangeAction["Replace"] = 2] = "Replace";
    ReactiveListChangeAction[ReactiveListChangeAction["Move"] = 3] = "Move";
    ReactiveListChangeAction[ReactiveListChangeAction["Reset"] = 4] = "Reset";
})(ReactiveListChangeAction || (ReactiveListChangeAction = {}));
export class ReactiveList extends ReactiveState {
    constructor(owner, items = [], errorScheduler) {
        super(errorScheduler);
        this.owner = owner;
        this.items = items;
    }
    wrapArrayFunction(func, args, eventArgs) {
        const sub = this.changing.subscribe(x => {
            if (x.value === eventArgs) {
                func.apply(x.source.items, args);
                x.source.notifyPropertyChanged(() => new ReactiveEvent(x.source, x.value));
                sub.unsubscribe();
            }
        });
        this.notifyPropertyChanging(() => new ReactiveEvent(this, eventArgs));
    }
    get(index) {
        return this.items[index];
    }
    set(index, value) {
        const eventArgs = {
            action: ReactiveListChangeAction.Replace,
            newItems: [value],
            newStartingIndex: index,
            oldItems: this.items.slice(index, index + 1),
            oldStartingIndex: index,
        };
        this.wrapArrayFunction(() => { this.items[index] = value; }, null, eventArgs);
    }
    asArray() {
        return this.items;
    }
    toArray() {
        return this.items.slice();
    }
    clear() {
        this.reset();
    }
    reset(...items) {
        this.splice(0, this.items.length, ...items);
    }
    // Array<TValue> Implementation
    get length() {
        return this.items.length;
    }
    push(...items) {
        this.wrapArrayFunction(this.items.push, arguments, {
            action: ReactiveListChangeAction.Add,
            newItems: items,
            newStartingIndex: this.items.length,
        });
    }
    pop() {
        this.wrapArrayFunction(this.items.pop, arguments, {
            action: ReactiveListChangeAction.Remove,
            oldItems: this.items.slice(-1),
            oldStartingIndex: this.items.length - 1,
        });
    }
    concat(...items) {
        return this.items.concat.apply(this.items, arguments);
    }
    join(separator) {
        return this.items.join.apply(this.items, arguments);
    }
    reverse() {
        this.wrapArrayFunction(this.items.reverse, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    }
    shift() {
        this.wrapArrayFunction(this.items.shift, arguments, {
            action: ReactiveListChangeAction.Remove,
            oldItems: this.items.slice(0, 1),
            oldStartingIndex: 0,
        });
    }
    slice(start, end) {
        return this.items.slice.apply(this.items, arguments);
    }
    sort(compareFn) {
        this.wrapArrayFunction(this.items.sort, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    }
    splice() {
        // we have to reset here because this could be an add, a remove, a replace, or some combination
        this.wrapArrayFunction(this.items.splice, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    }
    unshift(...items) {
        this.wrapArrayFunction(this.items.unshift, arguments, {
            action: ReactiveListChangeAction.Add,
            newItems: items,
            newStartingIndex: 0,
        });
    }
    indexOf(searchElement, fromIndex) {
        return this.items.indexOf.apply(this.items, arguments);
    }
    lastIndexOf(searchElement, fromIndex) {
        return this.items.lastIndexOf.apply(this.items, arguments);
    }
    every(callbackfn, thisArg) {
        return this.items.every.apply(this.items, arguments);
    }
    some(callbackfn, thisArg) {
        return this.items.some.apply(this.items, arguments);
    }
    forEach(callbackfn, thisArg) {
        return this.items.forEach.apply(this.items, arguments);
    }
    map(callbackfn, thisArg) {
        return this.items.map.apply(this.items, arguments);
    }
    filter(callbackfn, thisArg) {
        return this.items.filter.apply(this.items, arguments);
    }
    reduce(callbackfn, initialValue) {
        return this.items.reduce.apply(this.items, arguments);
    }
    reduceRight(callbackfn, initialValue) {
        return this.items.reduceRight.apply(this.items, arguments);
    }
    // Array<T> ES6 Shims
    find(predicate, thisArg) {
        return this.items.find.apply(this.items, arguments);
    }
    findIndex(predicate, thisArg) {
        return this.items.findIndex.apply(this.items, arguments);
    }
    fill(value, start, end) {
        // we have to reset here because this could be an add, a replace, or both
        this.wrapArrayFunction(this.items.fill, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    }
    copyWithin(target, start, end) {
        // we have to reset here because this could be an add, a replace, or both
        this.wrapArrayFunction(this.items.copyWithin, arguments, {
            action: ReactiveListChangeAction.Reset,
        });
    }
    entries() {
        return this.items.entries.apply(this.items, arguments);
    }
    keys() {
        return this.items.keys.apply(this.items, arguments);
    }
    values() {
        return this.items.values.apply(this.items, arguments);
    }
}

//# sourceMappingURL=ReactiveList.js.map
