import { ReactiveState } from './ReactiveState';
import { ReactiveEvent } from './ReactiveEvent';
import { ReactiveStreamProperty, ReactiveValueProperty } from './ReactiveProperty';
import { ReactiveCommand } from './ReactiveCommand';
import { ReactiveList } from './ReactiveList';
export class ReactiveObject extends ReactiveState {
    constructor(errorScheduler) {
        super(errorScheduler);
        this.members = [];
    }
    getMemberName(member) {
        if (member.name == null) {
            this.resolvePropertyNames();
        }
        return member.name;
    }
    resolvePropertyNames() {
        Object
            .getOwnPropertyNames(this)
            .map(name => ({ name, member: this[name] }))
            .filter(x => x.member != null && x.member.isReactive === true && x.member.name == null)
            .forEach(x => {
            x.member.name = x.name;
        });
    }
    registerMember(member) {
        this.add(member.changing
            .subscribe(x => {
            this.notifyPropertyChanging(() => new ReactiveEvent(this, {
                member: x.source,
                memberName: this.getMemberName(x.source),
            }));
        }, this.thrownErrorsHandler.next));
        this.add(member.changed
            .subscribe(x => {
            this.notifyPropertyChanged(() => new ReactiveEvent(this, {
                member: x.source,
                memberName: this.getMemberName(x.source),
            }));
        }, this.thrownErrorsHandler.next));
        this.members.push(member);
    }
    propertyFrom(source, initialValue, errorScheduler) {
        const prop = new ReactiveStreamProperty(source, this, initialValue, errorScheduler);
        this.registerMember(prop);
        return prop;
    }
    property(initialValue, scheduler, errorScheduler) {
        const prop = new ReactiveValueProperty(this, initialValue, scheduler, errorScheduler);
        this.registerMember(prop);
        return prop;
    }
    command(executeAction, canExecute, errorScheduler) {
        const cmd = new ReactiveCommand(this, executeAction, canExecute, errorScheduler);
        this.registerMember(cmd);
        return cmd;
    }
    list(items, errorScheduler) {
        const list = new ReactiveList(this, items, errorScheduler);
        this.registerMember(list);
        return list;
    }
}

//# sourceMappingURL=ReactiveObject.js.map
