declare module "PausableBuffer" {
    import { Observable } from 'rxjs';
    export function pausableBuffer<T>(isPausedObservable: Observable<boolean>, closeBuffer?: (buffer: T[]) => T[]): Observable<T>;
    export interface PausableBufferSignature<T> {
        (isPausedObservable: Observable<boolean>, closeBuffer?: (buffer: T[]) => T[]): Observable<T>;
    }
}
declare module "Extensions" {
    import { PausableBufferSignature } from "PausableBuffer";
    module '~rxjs/Observable' {
        interface Observable<T> {
            pausableBuffer: PausableBufferSignature<T>;
        }
    }
}
declare module "ReactiveApp" {
    import { Subscriber } from 'rxjs';
    import { QueueScheduler } from 'rxjs/scheduler/QueueScheduler';
    export class ReactiveApp {
        static isUnitTestRunner: boolean;
        static defaultErrorHandler: Subscriber<Error>;
        static mainScheduler: QueueScheduler;
        private static createMainScheduler();
    }
}
declare module "SubjectScheduler" {
    import { Subject, Subscription, Observable } from 'rxjs';
    import { Subscribable } from 'rxjs/Observable';
    import { Scheduler } from 'rxjs/Scheduler';
    import { PartialObserver } from 'rxjs/Observer';
    export class SubjectScheduler<T> extends Subscription implements Subscribable<T> {
        protected scheduler: Scheduler;
        protected defaultObserverOrNext: PartialObserver<T> | ((value: T) => void);
        protected subject: Subject<T>;
        constructor(scheduler: Scheduler, defaultObserverOrNext?: PartialObserver<T> | ((value: T) => void), subject?: Subject<T>);
        protected defaultObserverSub: Subscription;
        protected observerRefCount: number;
        next(value?: T): void;
        error(err: any): void;
        complete(): void;
        asObservable(): Observable<T>;
        subscribe(observerOrNext: PartialObserver<T> | ((value: T) => void), error?: (error: any) => void, complete?: () => void): Subscription;
    }
}
declare module "ReactiveState" {
    import { Subscription, Observable, Subject } from 'rxjs';
    import { Scheduler } from 'rxjs/Scheduler';
    import { SubjectScheduler } from "SubjectScheduler";
    export class ReactiveState<T> extends Subscription {
        constructor(errorScheduler?: Scheduler);
        private changeNotificationsSuppressed;
        private changeNotificationsDelayed;
        private startDelayNotificationsSubject;
        protected changingSubject: Subject<T>;
        protected changedSubject: Subject<T>;
        protected thrownErrorsHandler: SubjectScheduler<Error>;
        protected changingObservable: Observable<T>;
        protected changedObservable: Observable<T>;
        protected notifyPropertyChanging(changing: () => T): void;
        protected notifyPropertyChanged(changed: () => T): void;
        protected notifyObservable(change: () => T, subject: Subject<T>): void;
        isReactive: boolean;
        changing: Observable<T>;
        changed: Observable<T>;
        thrownErrors: Observable<Error>;
        areChangeNotificationsEnabled(): boolean;
        areChangeNotificationsDelayed(): boolean;
        suppressChangeNotifications(): Subscription;
        delayChangeNotifications(): Subscription;
    }
}
declare module "ReactiveEvent" {
    export class ReactiveEvent<TSource, TValue> {
        source: TSource;
        value: TValue;
        constructor(source: TSource, value: TValue);
    }
}
declare module "ReactiveProperty" {
    import { Observable } from 'rxjs';
    import { Scheduler } from 'rxjs/Scheduler';
    import { SubjectScheduler } from "SubjectScheduler";
    import { ReactiveEvent } from "ReactiveEvent";
    import { ReactiveState } from "ReactiveState";
    export abstract class ReactiveProperty<TObj, T> extends ReactiveState<ReactiveEvent<ReactiveProperty<TObj, T>, T>> {
        owner: TObj;
        constructor(owner: TObj, initialValue?: T, errorScheduler?: Scheduler);
        protected currentValue: T;
        protected initialize(source: Observable<T>): void;
        value: T;
    }
    export class ReactiveStreamProperty<TObj, T> extends ReactiveProperty<TObj, T> {
        source: Observable<T>;
        constructor(source: Observable<T>, owner: TObj, initialValue?: T, errorScheduler?: Scheduler);
    }
    export class ReactiveValueProperty<TObj, T> extends ReactiveProperty<TObj, T> {
        constructor(owner: TObj, initialValue?: T, scheduler?: Scheduler, errorScheduler?: Scheduler);
        protected valueHandler: SubjectScheduler<T>;
        source: Observable<T>;
        value: T;
    }
}
declare module "ReactiveList" {
    import { Scheduler } from 'rxjs/Scheduler';
    import { ReactiveEvent } from "ReactiveEvent";
    import { ReactiveState } from "ReactiveState";
    global  {
        interface Array<T> {
            toReactiveList<TObject>(owner: TObject, errorScheduler?: Scheduler): ReactiveList<TObject, T>;
        }
    }
    export enum ReactiveListChangeAction {
        Add = 0,
        Remove = 1,
        Replace = 2,
        Move = 3,
        Reset = 4,
    }
    export interface ReactiveListEventValue<TValue> {
        action: ReactiveListChangeAction;
        newItems?: TValue[];
        oldItems?: TValue[];
        newStartingIndex?: number;
        oldStartingIndex?: number;
    }
    export class ReactiveList<TObject, TValue> extends ReactiveState<ReactiveEvent<ReactiveList<TObject, TValue>, ReactiveListEventValue<TValue>>> {
        owner: TObject;
        constructor(owner: TObject, items?: TValue[], errorScheduler?: Scheduler);
        private items;
        private wrapArrayFunction<TResult>(func, args, eventArgs);
        get(index: number): TValue;
        set(index: number, value: TValue): void;
        asArray(): TValue[];
        toArray(): TValue[];
        clear(): void;
        reset(...items: TValue[]): void;
        length: number;
        push(...items: TValue[]): void;
        pop(): void;
        concat<T extends TValue[]>(...items: T[]): TValue[];
        join(separator?: string): string;
        reverse(): void;
        shift(): void;
        slice(start?: number, end?: number): TValue[];
        sort(compareFn?: (a: TValue, b: TValue) => number): void;
        splice(start: number): void;
        splice(start: number, deleteCount: number, ...items: TValue[]): void;
        unshift(...items: TValue[]): void;
        indexOf(searchElement: TValue, fromIndex?: number): number;
        lastIndexOf(searchElement: TValue, fromIndex?: number): number;
        every(callbackfn: (value: TValue, index: number, array: TValue[]) => boolean, thisArg?: any): boolean;
        some(callbackfn: (value: TValue, index: number, array: TValue[]) => boolean, thisArg?: any): boolean;
        forEach(callbackfn: (value: TValue, index: number, array: TValue[]) => void, thisArg?: any): void;
        map<T>(callbackfn: (value: TValue, index: number, array: TValue[]) => T, thisArg?: any): T[];
        filter(callbackfn: (value: TValue, index: number, array: TValue[]) => boolean, thisArg?: any): TValue[];
        reduce<T>(callbackfn: (previousValue: T, currentValue: TValue, currentIndex: number, array: TValue[]) => T, initialValue: T): T;
        reduceRight<T>(callbackfn: (previousValue: T, currentValue: TValue, currentIndex: number, array: TValue[]) => T, initialValue: T): T;
        find(predicate: (value: TValue, index: number, obj: Array<TValue>) => boolean, thisArg?: any): TValue;
        findIndex(predicate: (value: TValue) => boolean, thisArg?: any): number;
        fill(value: TValue, start?: number, end?: number): void;
        copyWithin(target: number, start: number, end?: number): void;
        entries(): Iterator<[number, TValue]>;
        keys(): Iterator<number>;
        values(): Iterator<TValue>;
    }
}
declare module "ReactiveCommand" {
    import { Observable } from 'rxjs';
    import { Scheduler } from 'rxjs/Scheduler';
    import { ReactiveEvent } from "ReactiveEvent";
    import { ReactiveState } from "ReactiveState";
    export interface ReactiveCommandEventValue<T> {
        param: any;
        result: T;
    }
    export class ReactiveCommand<TObj, TResult> extends ReactiveState<ReactiveEvent<ReactiveCommand<TObj, TResult>, ReactiveCommandEventValue<TResult>>> {
        owner: TObj;
        protected executeAction: (param: any) => TResult;
        constructor(owner: TObj, executeAction: (param: any) => TResult, canExecute?: Observable<boolean>, scheduler?: Scheduler, errorScheduler?: Scheduler);
        private isExecutingSubject;
        private isExecutingScheduledSubject;
        private canExecuteObservable;
        isExecuting: Observable<boolean>;
        results: Observable<TResult>;
        canExecute: Observable<boolean>;
        execute(param?: any): void;
    }
}
declare module "ReactiveObject" {
    import { Observable } from 'rxjs';
    import { Scheduler } from 'rxjs/Scheduler';
    import { ReactiveState } from "ReactiveState";
    import { ReactiveEvent } from "ReactiveEvent";
    import { ReactiveStreamProperty, ReactiveValueProperty } from "ReactiveProperty";
    import { ReactiveCommand } from "ReactiveCommand";
    import { ReactiveList } from "ReactiveList";
    export type ReactiveMember = ReactiveState<ReactiveEvent<ReactiveState<any>, any>>;
    export interface ReactivePropertyEventValue {
        member: ReactiveState<any>;
        memberName: string;
    }
    export abstract class ReactiveObject extends ReactiveState<ReactiveEvent<ReactiveObject, ReactivePropertyEventValue>> {
        constructor(errorScheduler?: Scheduler);
        protected members: ReactiveMember[];
        private getMemberName(member);
        private resolvePropertyNames();
        protected registerMember(member: ReactiveMember): void;
        protected propertyFrom<T>(source: Observable<T>, initialValue?: T, errorScheduler?: Scheduler): ReactiveStreamProperty<this, T>;
        protected property<T>(initialValue?: T, scheduler?: Scheduler, errorScheduler?: Scheduler): ReactiveValueProperty<this, T>;
        protected command<TResult>(executeAction: (param: any) => TResult, canExecute?: Observable<boolean>, errorScheduler?: Scheduler): ReactiveCommand<this, TResult>;
        protected list<TValue>(items?: TValue[], errorScheduler?: Scheduler): ReactiveList<this, TValue>;
    }
}
declare module "rxobj" {
    import "Extensions";
    export { ReactiveApp } from "ReactiveApp";
    export { ReactiveState } from "ReactiveState";
    export { ReactiveEvent } from "ReactiveEvent";
    export { ReactiveProperty } from "ReactiveProperty";
    export { ReactiveList } from "ReactiveList";
    export { ReactiveObject } from "ReactiveObject";
}
